from pynetfilter_conntrack import ConntrackEntry, Filter, \
    nfct_query, nfct_callback_t, nfct_callback_register, \
    nfct_callback_unregister, nfct_catch, \
    CONNTRACK, NFCT_Q_DUMP, NFCT_T_ALL, NFCT_CB_CONTINUE, NFCT_CB_STOLEN
from pynetfilter_conntrack.conntrack_base import ConntrackBase
from ctypes import byref
from pynetfilter_conntrack.ctypes_stdint import uint8_t
from socket import AF_INET
try:
    from cnetfilter_conntrack import dump_table_ipv4
    HAS_CNETFILTER_CONNTRACK = True
except ImportError, err:
    HAS_CNETFILTER_CONNTRACK = False
from IPy import IP

class Conntrack(ConntrackBase):
    def __init__(self, subscriptions=0, subsys=CONNTRACK):
        ConntrackBase.__init__(self, subsys, subscriptions)

    def register_callback(self, callback, event_type=NFCT_T_ALL, data=None):
        """
        Register a callback, needed by some query.
        Callback prototype is: func(msgtype, ct, data), the callback have to
        return: NFCT_CB_CONTINUE, NFCT_CB_FAILURE, NFCT_CB_STOP,
        or NFCT_CB_STOLEN (like continue, but ct is not freed).
        """
        self.callback = nfct_callback_t(callback)
        self.callback_arg = data
        nfct_callback_register(self.handle, event_type, self.callback, self.callback_arg)

    def unregister_callback(self):
        """Unregister callback"""
        nfct_callback_unregister(self.handle)
        self.callback = None
        self.callback_arg = None

    def dump_table(self, family=AF_INET, event_type=NFCT_T_ALL, filter=None):
        if not filter:
            filter = Filter()
        if HAS_CNETFILTER_CONNTRACK:
            if family != AF_INET:
                raise ValueError("cnetfilter_conntrack only supports IPv4")
            options = filter.createCNetfilterOptions()
            table, total = dump_table_ipv4(self.handle, **options)

            connections = []
            for attr in table:
                handle = attr.pop('handle')
                for key, value in attr.iteritems():
                    if "ipv4" in key:
                        attr[key] = IP(value)
                conn = ConntrackEntry(self, handle, attr=attr)
                connections.append(conn)

            return connections, total
        else:
            # Create a pointer to a 'uint8_t' of the address family
            family = byref(uint8_t(family))

            def copyEntry(msgtype, conntrack, data):
                conn = ConntrackEntry(self, conntrack, msgtype)
                if not filter.filterConnection(conn):
                    conn._destroy = False
                    return NFCT_CB_CONTINUE
                copyEntry.ctlist.append(conn)
                return NFCT_CB_STOLEN
            copyEntry.ctlist = []

            # Install callback, do the query, remove callback
            self.register_callback(copyEntry, event_type)
            self.query(NFCT_Q_DUMP, family)
            self.unregister_callback()
            connset = copyEntry.ctlist

            # Sort the list
            filter.sortTable(connset)

            # Truncated the list
            total = len(connset)
            connset = filter.truncate(connset)

            # Suppress unwanted entries
            return connset, total

    def query(self, command, argument):
        """
        Do query libnetfilter_conntrack:
         - command: NFCT_Q_CREATE, NFCT_Q_UPDATE, ...
         - argument (optional): value depends on command

        May raise a RuntimeError.
        """
        ret = nfct_query(self.handle, command, argument)
        if ret != 0:
            self._error('nfct_query')

    def catch(self, callback):
        """
        Catch all Netfilter events: call specified callback for each event.
        See register_callback() method for callback details.
        """
        self.register_callback(callback)
        ret = nfct_catch(self.handle)
        if ret != 0:
            self._error('nfct_catch')
        self.unregister_callback()

__all__ = ("Conntrack", "HAS_CNETFILTER_CONNTRACK")




from pynetfilter_conntrack import NFCT_Q_CREATE, NFCT_T_UNKNOWN
from pynetfilter_conntrack.ctypes_errno import get_errno
from os import strerror

BUFFER_SIZE = 1024 # bytes including nul byte

class EntryBase(object):
    def __init__(self, conntrack, handle, msgtype=NFCT_T_UNKNOWN, destroy=True, attr=None):
        if not handle:
            raise RuntimeError("Empty entry handler")
        if not attr:
            attr = {}
        object.__setattr__(self, "_attr", attr)
        object.__setattr__(self, "_destroy", destroy)
        object.__setattr__(self, "_handle", handle)
        object.__setattr__(self, "_sub_system", conntrack)
        object.__setattr__(self, "_msgtype", msgtype)

    def _error(self, func_name):
        errno = get_errno()
        err_msg = strerror(errno)
        raise RuntimeError("%s() failure: %s" % (func_name, err_msg))

    def query(self, command):
        self._sub_system.query(command, self._handle)

    def create(self):
        self.query(NFCT_Q_CREATE)

    def __str__(self):
        return self.format()

    def __del__(self):
        if '_destroy' not in self.__dict__ or not self._destroy:
            return
        if '_handle' not in self.__dict__ or not self._handle:
            return
        self._free()
        object.__setattr__(self, '_handle', None)

    # --- Abstract methods ---

    def format(self, **kw):
        raise NotImplementedError()

    def free(self):
        raise NotImplementedError()

__all__ = ('EntryBase', 'BUFFER_SIZE')




from pynetfilter_conntrack import \
    nfexp_new, nfexp_destroy, nfexp_snprintf,\
    NFCT_O_DEFAULT, NFCT_OF_SHOW_LAYER3
from ctypes import create_string_buffer
from pynetfilter_conntrack.entry_base import EntryBase, BUFFER_SIZE

class ExpectEntry(EntryBase):
    @staticmethod
    def new(expect):
        handle = nfexp_new()
        return ExpectEntry(expect, handle)

    def _free(self):
        nfexp_destroy(self._handle)

    def format(self, msg_output=NFCT_O_DEFAULT, msgtype=None, flags=NFCT_OF_SHOW_LAYER3):
        """
        Format the entry:
         - msgtype: NFCT_T_UNKNOWN, NFCT_T_NEW, NFCT_T_UPDATE,
                    NFCT_T_DESTROY or NFCT_T_ERROR
         - msg_output: NFCT_O_DEFAULT or NFCT_O_XML
         - flags: 0 or NFCT_OF_SHOW_LAYER3

        Return a string.

        Raise a RuntimeError on error.
        """
        buffer = create_string_buffer(BUFFER_SIZE)
        if msgtype is None:
            msgtype = self._msgtype
        ret = nfexp_snprintf(buffer, BUFFER_SIZE, self._handle, msgtype, msg_output, flags)
        if not(0 <= ret <= (BUFFER_SIZE-1)):
            self._error('nfct_snprintf')
        return buffer.value

__all__ = ('ExpectEntry',)




from pynetfilter_conntrack import \
    nfct_open, nfct_close
from pynetfilter_conntrack.ctypes_errno import get_errno
from os import strerror

class ConntrackBase(object):
    def __init__(self, subsys, subscriptions):
        """
        Create new conntrack object. May raise a RuntimeError.
        """
        # Callback things
        self.callback = None
        self.callback_arg = None

        # Default value, needed by __del__ if an exception is raised in the constructor
        self.conntrack = None
        self.handle = None

        # Open a conntrack handler
        self.handle = nfct_open(subsys, subscriptions)
        if not self.handle:
            self._error('nfct_new')

    def _error(self, func_name):
        errno = get_errno()
        err_msg = strerror(errno)
        raise RuntimeError("%s() failure: %s" % (func_name, err_msg))

    def __del__(self):
        """Destroy conntrack object"""
        if 'handle' not in self.__dict__:
            return
        self.close()

    def close(self):
        if not self.handle:
            return
        nfct_close(self.handle)
        self.handle = None

__all__ = ('ConntrackBase',)




from pynetfilter_conntrack import \
    IPPROTO_TCP, TCP_CONNTRACK_TIME_WAIT, PF_INET, PF_INET6

class Filter:
    def __init__(self):
        self.start = 0
        self.size = None
        self.reverse = False
        self.sort = "orig_ipv4_src"
        self.drop_networks = None

    def filterConnection(self, conn):
        # Ignore TCP connection in state TIME_WAIT
        if (conn.orig_l4proto == IPPROTO_TCP) \
        and (conn.tcp_state == TCP_CONNTRACK_TIME_WAIT):
            return False

        # Get source and destination IP (v4 or v6) addresses
        l3proto = conn.orig_l3proto
        if l3proto == PF_INET:
            ip_src = conn.orig_ipv4_src
            ip_dst = conn.orig_ipv4_dst
        elif l3proto == PF_INET6:
            ip_src = conn.orig_ipv6_src
            ip_dst = conn.orig_ipv6_dst
        else:
            return True

        # Ignore IP address in self.filter
        if self.drop_networks:
            for network in self.drop_networks:
                if (ip_src in network) or (ip_dst in network):
                    return False
        return True

    def createCNetfilterOptions(self):
        if not self.size:
            size = 0
        else:
            size = self.size
        options = {
            'start': self.start,
            'size': size,
            'reverse': self.reverse,
            'sort': self.sort,
        }
        if self.drop_networks:
            options['drop_networks'] = tuple((ip.int(), ip.broadcast().int()) for ip in self.drop_networks)
        return options

    def truncate(self, connset):
        if self.size is not None:
            return connset[self.start:self.start + self.size]
        else:
            return connset[self.start:]

    def sortKey(self, conn):
        return getattr(conn, self.sort)

    def sortTable(self, connset):
        connset.sort(key=self.sortKey, reverse=self.reverse)




from ctypes import Structure, cdll, POINTER, CFUNCTYPE,\
    c_int, c_uint, c_void_p, c_char_p, util
from pynetfilter_conntrack.ctypes_stdint import uint8_t, uint16_t, uint32_t

# Open the dynamic library
library = cdll.LoadLibrary(util.find_library("netfilter_conntrack"))

#--------------------------------------------------------------------------
# Define nf_conntrack_p and nfct_handle_p: pointer to fake structures
#
#class nf_conntrack(Structure):
#    pass
#class nfct_handle(Structure):
#    pass
#
#nf_conntrack_p = POINTER(nf_conntrack)
#nfct_handle_p = POINTER(nfct_handle)
nf_conntrack_p = c_int
nfct_handle_p = c_int

class nfct_conntrack_compare_t(Structure):
    _fields_ = (
        ("ct", nf_conntrack_p),
        ("flags", c_uint),
        ("l3flags", c_uint),
        ("l4flags", c_uint),
    )
nfct_conntrack_compare_p = POINTER(nfct_conntrack_compare_t)

#--------------------------------------------------------------------------
# int callback(enum nf_conntrack_msg_type type,
#              struct nf_conntrack *ct,
#              void *data)
#
# Callback type
nfct_callback_t = CFUNCTYPE(c_int, c_int, nf_conntrack_p, c_void_p)

#--------------------------------------------------------------------------
# struct nf_conntrack *nfct_new(void): Allocate a new conntrack
#
# In case of success, this function returns a valid pointer to a memory blob,
# otherwise NULL is returned and errno is set appropiately."""
nfct_new = library.nfct_new
nfct_new.argtypes = None
nfct_new.restype = nf_conntrack_p

#--------------------------------------------------------------------------
# void nfct_destroy(struct nf_conntrack *ct): Release a conntrack object
#
# @ct: pointer to the conntrack object
nfct_destroy = library.nfct_destroy
nfct_destroy.argtypes = (nf_conntrack_p, )
nfct_destroy.restype = None

#--------------------------------------------------------------------------
# struct nfct_handle *nfct_open(u_int8_t subsys_id, unsigned subscriptions)
#
# Open a conntrack handler
nfct_open = library.nfct_open
nfct_open.argtypes = (uint8_t, c_uint)
nfct_open.restype = nfct_handle_p

#--------------------------------------------------------------------------
# int nfct_close(struct nfct_handle *cth)
#
# Open a conntrack handler
nfct_close = library.nfct_close
nfct_close.argtypes = (nfct_handle_p,)
nfct_close.restype = c_int

#--------------------------------------------------------------------------
# int nfct_query(struct nfct_handle *h,
#                const enum nf_conntrack_query qt,
#                const void *data)
#
# Send a query to ctnetlin
#    @h: library handler
#    @qt: query type
#    @data: data required to send the query
#
# On error, -1 is returned and errno is explicitely set. On success, 0
# is returned
nfct_query = library.nfct_query
nfct_query.argtypes = (nfct_handle_p, c_int, c_void_p)
nfct_query.restype = c_int

#--------------------------------------------------------------------------
# int nfct_callback_register(struct nfct_handle *h,
#                            enum nf_conntrack_msg_type type,
#                            int (*cb)(enum nf_conntrack_msg_type type,
#                                      struct nf_conntrack *ct,
#					                   void *data),
#				             void *data);
# Register callback.
nfct_callback_register = library.nfct_callback_register
nfct_callback_register.argtypes = (nfct_handle_p, c_int, nfct_callback_t, c_void_p)
nfct_callback_register.restype = c_int

#--------------------------------------------------------------------------
# void nfct_callback_unregister(struct nfct_handle *h)
#
# Unregister callback
nfct_callback_unregister = library.nfct_callback_unregister
nfct_callback_unregister.argtypes = (nfct_handle_p,)
nfct_callback_unregister.restype = None

#--------------------------------------------------------------------------
# int nfct_snprintf(char *buf,
#                   unsigned int size,
#                   const struct nf_conntrack *ct,
#                   const unsigned int msg_type,
#                   const unsigned int out_type,
#                   const unsigned int out_flags);
#
# Format a conntrack entry to a string
nfct_snprintf = library.nfct_snprintf
nfct_snprintf.argtypes = (c_char_p, c_uint, nf_conntrack_p, c_uint, c_uint, c_uint)
nfct_snprintf.restype = c_int

#--------------------------------------------------------------------------
# struct nf_conntrack *nfct_clone(const struct nf_conntrack *ct)
# nfct_clone - clone a conntrack object
# @ct: pointer to a valid conntrack object
#
# On error, NULL is returned and errno is appropiately set. Otherwise,
# a valid pointer to the clone conntrack is returned.
nfct_clone = library.nfct_clone
nfct_clone.argtypes = (nf_conntrack_p,)
nfct_clone.restype = nf_conntrack_p

#--------------------------------------------------------------------------
# const void *nfct_get_attr(const struct nf_conntrack *ct,
#				 const enum nf_conntrack_attr type);
# u_int8_t nfct_get_attr_u8(const struct nf_conntrack *ct,
#				 const enum nf_conntrack_attr type);
# u_int16_t nfct_get_attr_u16(const struct nf_conntrack *ct,
#				   const enum nf_conntrack_attr type);
# u_int32_t nfct_get_attr_u32(const struct nf_conntrack *ct,
#				   const enum nf_conntrack_attr type);
#
# Get an attribute on a conntrack entry
def getter(restype, suffix):
    func = getattr(library, "nfct_get_attr" + suffix)
    func.argtypes = (nf_conntrack_p, c_uint)
    func.restype = restype
    return func

nfct_get_attr = getter(c_void_p, "")
nfct_get_attr_u8 = getter(uint8_t, "_u8")
nfct_get_attr_u16 = getter(uint16_t, "_u16")
nfct_get_attr_u32 = getter(uint32_t, "_u32")
del getter


#--------------------------------------------------------------------------
# void nfct_set_attr(struct nf_conntrack *ct,
#                    const enum nf_conntrack_attr type,
#                    void *value);
#
# void nfct_set_attr_u8(struct nf_conntrack *ct,
#                       const enum nf_conntrack_attr type,
#                       u_int8_t value);
#
# void nfct_set_attr_u16(struct nf_conntrack *ct,
#                        const enum nf_conntrack_attr type,
#                        u_int16_t value);
#
# void nfct_set_attr_u32(struct nf_conntrack *ct,
#                        const enum nf_conntrack_attr type,
#                        u_int32_t value);
#
# Set an attribute on a conntrack entry
def setter(argtype, suffix):
    func = getattr(library, "nfct_set_attr" + suffix)
    func.argtypes = (nf_conntrack_p, argtype)
    func.restype = None
    return func

nfct_set_attr = setter(c_void_p, "")
nfct_set_attr_u8 = setter(uint8_t, "_u8")
nfct_set_attr_u16 = setter(uint16_t, "_u16")
nfct_set_attr_u32 = setter(uint32_t, "_u32")
del setter

# -------------------------------------------------------------------------
# int nfct_catch(struct nfct_handle *h);
nfct_catch = library.nfct_catch
nfct_catch.argtypes = (nfct_handle_p,)
nfct_catch.restype = c_int

# -------------------------------------------------------------------------
# int nfct_setobjopt(struct nf_conntrack *ct, unsigned int option);
nfct_setobjopt = library.nfct_setobjopt
nfct_setobjopt.argtypes = (nf_conntrack_p, c_uint)
nfct_setobjopt.restype = c_int

# -------------------------------------------------------------------------
# int nfct_conntrack_compare(struct nfct_conntrack *ct1,
#                            struct nfct_conntrack *ct2,
#                            struct nfct_conntrack_compare *cmp);
nfct_conntrack_compare = library.nfct_cmp
nfct_conntrack_compare.argtypes = (nf_conntrack_p, nf_conntrack_p, nfct_conntrack_compare_p)
nfct_conntrack_compare.restype = c_int

# -------------------------------------------------------------------------

__all__ = (
    "nfct_handle_p", "nfct_callback_t",
    "nfct_new", "nfct_destroy", "nfct_open", "nfct_close", "nfct_query",
    "nfct_callback_register", "nfct_callback_unregister",
    "nfct_clone", "nfct_snprintf", "nfct_catch", "nfct_setobjopt",
    "nfct_get_attr", "nfct_get_attr_u8", "nfct_get_attr_u16", "nfct_get_attr_u32",
    "nfct_set_attr", "nfct_set_attr_u8", "nfct_set_attr_u16", "nfct_set_attr_u32",
    "nfct_conntrack_compare_t", "nfct_conntrack_compare",
)




from ctypes import POINTER, c_int, pythonapi, py_object

# Function from pypy project:
# File pypy/dist/pypy/rpython/rctypes/aerrno.py
def _pythonapi_geterrno():
    """
    Read errno using Python C API: raise an exception with PyErr_SetFromErrno
    and then read error code 'errno'.

    This function may raise an RuntimeError.
    """
    try:
        pythonapi.PyErr_SetFromErrno(py_object(OSError))
    except OSError, err:
        return err.errno
    else:
        raise RuntimeError("get_errno() is unable to get error code")

def _errno_location():
    """
    Try to get errno integer from libc using __errno_location_sym function.

    This function is specific to OS with "libc.so.6" and may fails for
    thread-safe libc.
    """
    from ctypes import cdll
    try:
        libc = cdll.LoadLibrary("libc.so.6")
    except OSError:
        # Unable to open libc dynamic library
        return None
    try:
        __errno_location = libc.__errno_location_sym
    except AttributeError:
        # libc doesn't have __errno_location
        return None
    __errno_location.restype = POINTER(c_int)
    return __errno_location()[0]

errno = _errno_location()
if errno is not None:
    get_errno = lambda: errno
else:
    del errno
    get_errno = _pythonapi_geterrno

__all__ = ["get_errno"]




from pynetfilter_conntrack import \
    nfct_new, nfct_destroy, nfct_snprintf,\
    nfct_get_attr, nfct_get_attr_u8, nfct_get_attr_u16, nfct_get_attr_u32,\
    nfct_set_attr, nfct_set_attr_u8, nfct_set_attr_u16, nfct_set_attr_u32,\
    nfct_setobjopt,\
    NFCT_O_DEFAULT, NFCT_OF_SHOW_LAYER3,\
    ATTRIBUTES, NFCT_Q_UPDATE, \
    NFCT_Q_DESTROY, NFCT_MARK, \
    nfct_conntrack_compare_t, nfct_conntrack_compare,\
    ctypes_ptr2uint, int16_to_uint16, int32_to_uint32
from ctypes import create_string_buffer
from socket import ntohs, ntohl, htons, htonl
from IPy import IP
from pynetfilter_conntrack.entry_base import EntryBase, BUFFER_SIZE

IP_ATTRIBUTES = set((
    "orig_ipv4_src", "orig_ipv4_dst",
    "repl_ipv4_src", "repl_ipv4_dst",
    "orig_ipv6_src", "orig_ipv6_dst",
    "repl_ipv6_src", "repl_ipv6_dst",
))

GETTER = {
      8: nfct_get_attr_u8,
     16: nfct_get_attr_u16,
     32: nfct_get_attr_u32,
     64: nfct_get_attr,
    128: nfct_get_attr,
}
SETTER = {
      8: nfct_set_attr_u8,
     16: nfct_set_attr_u16,
     32: nfct_set_attr_u32,
     64: nfct_set_attr,
    128: nfct_set_attr,
}

HTON = {8: None, 16: htons, 32: htonl, 64: None, 128: None}

class ConntrackEntry(EntryBase):
    @staticmethod
    def new(conntrack):
        handle = nfct_new()
        return ConntrackEntry(conntrack, handle)

    def __getattr__(self, name):
        if name not in self._attr:
            self._attr[name] = self._getAttr(name)
        return self._attr[name]

    def _getAttr(self, name):
        try:
            attrid, nbits = ATTRIBUTES[name]
            getter = GETTER[nbits]
        except KeyError:
            raise AttributeError("ConntrackEntry object has no attribute '%s'" % name)
        value = getter(self._handle, attrid)
        if 32 < nbits:
            return ctypes_ptr2uint(value, nbits//8)

        if nbits in (16, 32) and name not in ("mark", "timeout", "status"):
            if nbits == 16:
                value = ntohs(value)
                value = int16_to_uint16(value)
            else:
                value = ntohl(value)
                value = int32_to_uint32(value)
        if name in IP_ATTRIBUTES:
            value = IP(value, ipversion=4)
        return value

    def _setAttr(self, name, value):
        try:
            attrid, nbits = ATTRIBUTES[name]
            setter = SETTER[nbits]
            hton = HTON[nbits]
        except KeyError:
            raise AttributeError("ConntrackEntry object has no attribute '%s'" % name)
        if hton and name not in ("mark", "timeout", "status"):
            value = hton(value)
        setter(self._handle, attrid, value)
        return value

    def __setattr__(self, name, value):
        if name in ATTRIBUTES:
            python_value = value
            if name in IP_ATTRIBUTES:
                if isinstance(value, IP):
                    value = value.int()
                else:
                    python_value = IP(value, ipversion=4)
            self._attr[name] = python_value
            self._setAttr(name, value)
        elif name.startswith('_'):
            object.__setattr__(self, name, value)
        else:
            raise AttributeError("ConntrackEntry object has no attribute '%s'" % name)

    def _free(self):
        """
        Destroy the conntrack entry: free the memory.
        Function called by the Python destructor.
        """
        nfct_destroy(self._handle)

    def destroy(self):
        """
        Destroy (kill) a connection in the conntrack table.
        """
        self.query(NFCT_Q_DESTROY)

    def format(self, msg_output=NFCT_O_DEFAULT, msgtype=None, flags=NFCT_OF_SHOW_LAYER3):
        """
        Format the entry:
         - msgtype: NFCT_T_UNKNOWN, NFCT_T_NEW, NFCT_T_UPDATE,
                    NFCT_T_DESTROY or NFCT_T_ERROR
         - msg_output: NFCT_O_DEFAULT or NFCT_O_XML
         - flags: 0 or NFCT_OF_SHOW_LAYER3

        Return a string.

        Raise a RuntimeError on error.
        """
        buffer = create_string_buffer(BUFFER_SIZE)
        if msgtype is None:
            msgtype = self._msgtype
        ret = nfct_snprintf(buffer, BUFFER_SIZE, self._handle, msgtype, msg_output, flags)
        if not(0 <= ret <= (BUFFER_SIZE-1)):
            self._error('nfct_snprintf')
        return buffer.value

    def update(self):
        self.query(NFCT_Q_UPDATE)

    def setobjopt(self, option):
        nfct_setobjopt(self._handle, option)

    def compare(self, other, use_layer3=True, use_layer4=True, use_mark=False):
        flags = 0
        if use_mark:
            flags |= NFCT_MARK
        cmp_struct = nfct_conntrack_compare_t(
            None, flags, int(use_layer3), int(use_layer4))
        return nfct_conntrack_compare(self._handle, other._handle, cmp_struct)

    def __eq__(self, other):
        return self.compare(other) == 0

__all__ = ("ConntrackEntry",)




#!/usr/bin/env python

from pynetfilter_conntrack import Conntrack, NFCT_O_DEFAULT, NFCT_O_XML
from socket import AF_INET
from subprocess import call
import sys

OUTPUT_FORMAT = {
    "list": NFCT_O_DEFAULT,
    "xml": NFCT_O_XML,
}

def checkKernelModule(module_name, symbol):
    allsyms = open('/proc/kallsyms')
    try:
        for line in allsyms:
            if symbol in line:
                print "Module %s is loaded: symbol %r is present" % (module_name, symbol)
                return
    finally:
        allsyms.close()
    print "Load kernel module %s" % module_name
    exitcode = call("modprobe %s" % (module_name), shell = True)
    if exitcode:
        raise RuntimeError("modprobe error (exit code %d)" % exitcode)

def main():
    if len(sys.argv) != 2 or sys.argv[1] not in OUTPUT_FORMAT:
        print >>sys.stderr, "usage: %s command" % sys.argv[0]
        print >>sys.stderr
        print >>sys.stderr, "command: list or xml"
        sys.exit(1)
    mode = sys.argv[1]
    output = OUTPUT_FORMAT[mode]

    checkKernelModule('nf_conntrack', 'nf_ct_attach')
    checkKernelModule('nf_conntrack_netlink', 'ctnetlink_net_init')
    try:
        if mode == "xml":
            print '<?xml version="1.0" encoding="ISO-8859-1"?>'
            print '<flows>'
        nf = Conntrack()
        (table, count) = nf.dump_table(AF_INET)
        for entry in table:
            print entry.format(output)
        if mode == "xml":
            print '</flows>'
    except RuntimeError, err:
        print "ERROR: %s" % err
    except KeyboardInterrupt:
        print "Interrupted."

if __name__ == "__main__":
    main()




from pynetfilter_conntrack import nfct_handle_p
from pynetfilter_conntrack.func import library
from ctypes import (Structure, POINTER, CFUNCTYPE,
    c_int, c_void_p, c_char_p, c_uint)
from pynetfilter_conntrack.ctypes_stdint import uint8_t, uint16_t, uint32_t

#--------------------------------------------------------------------------
# Define nf_expect: pointer to fake structures
class nf_expect(Structure):
    pass
nf_expect_p = POINTER(nf_expect)

nfexp_callback_t = CFUNCTYPE(c_int, c_int, nf_expect_p, c_void_p)

#--------------------------------------------------------------------------
# struct nf_expect *nfexp_new(void);
nfexp_new = library.nfexp_new
nfexp_new.argtypes = None
nfexp_new.restype = nf_expect_p

#--------------------------------------------------------------------------
# void nfexp_destroy(struct nf_expect *exp);
nfexp_destroy = library.nfexp_destroy
nfexp_destroy.argtypes = None
nfexp_destroy.restype = nf_expect_p

# -------------------------------------------------------------------------
# int nfexp_query(struct nfct_handle *h,
#                 const enum nf_conntrack_query qt,
#                 const void *data);
nfexp_query = library.nfexp_query
nfexp_query.argtypes = (nfct_handle_p, c_int, c_void_p)
nfexp_query.restype = c_int

#--------------------------------------------------------------------------
# int nfexp_callback_register(struct nfct_handle *h,
#                            enum nf_conntrack_msg_type type,
#                            int (*cb)(enum nf_conntrack_msg_type type,
#                                      struct nf_conntrack *ct,
#                                      void *data),
#                            void *data);
# Register callback.
nfexp_callback_register = library.nfexp_callback_register
nfexp_callback_register.argtypes = (nfct_handle_p, c_int, nfexp_callback_t, c_void_p)
nfexp_callback_register.restype = c_int

#--------------------------------------------------------------------------
# int nfexp_snprintf(char *buf,
#                    unsigned int size,
#                    const struct nf_expect *exp,
#                    const unsigned int msg_type,
#                    const unsigned int out_type,
#                    const unsigned int out_flags);
nfexp_snprintf = library.nfexp_snprintf
nfexp_snprintf.argtypes = (c_char_p, c_uint, nf_expect_p, c_uint, c_uint, c_uint)
nfexp_snprintf.restype = c_int

#--------------------------------------------------------------------------
# void nfexp_callback_unregister(struct nfct_handle *h);
nfexp_callback_unregister = library.nfexp_callback_unregister
nfexp_callback_unregister.argtypes = (nfct_handle_p,)
nfexp_callback_unregister.restype = None

# -------------------------------------------------------------------------
# int nfct_catch(struct nfct_handle *h);
nfexp_catch = library.nfexp_catch
nfexp_catch.argtypes = (nfct_handle_p,)
nfexp_catch.restype = c_int

# -------------------------------------------------------------------------
# void nfexp_set_attr(struct nf_expect *exp,
#                     const enum nf_expect_attr type,
#                     const void *value);
#
# void nfexp_set_attr_u8(struct nf_expect *exp,
#                        const enum nf_expect_attr type,
#                        u_int8_t value);
#
# void nfexp_set_attr_u16(struct nf_expect *exp,
#                         const enum nf_expect_attr type,
#                         u_int16_t value);
#
# void nfexp_set_attr_u32(struct nf_expect *exp,
#                         const enum nf_expect_attr type,
#                         u_int32_t value);
def setter(argtype, suffix):
    func = getattr(library, "nfexp_set_attr" + suffix)
    func.argtypes = (nf_expect_p, argtype)
    func.restype = None
    return func

nfexp_set_attr = setter(c_void_p, "")
nfexp_set_attr_u8 = setter(uint8_t, "_u8")
nfexp_set_attr_u16 = setter(uint16_t, "_u16")
nfexp_set_attr_u32 = setter(uint32_t, "_u32")
del setter

__all__ = (
    'nf_expect_p', 'nfexp_callback_t',
    'nfexp_new', 'nfexp_destroy',
    'nfexp_query', 'nfexp_callback_register', 'nfexp_snprintf',
    'nfexp_callback_unregister', 'nfexp_catch',
    'nfexp_set_attr',
    'nfexp_set_attr_u8', 'nfexp_set_attr_u16', 'nfexp_set_attr_u32',
)



from pynetfilter_conntrack import ExpectEntry,\
    nfexp_callback_t,\
    nfexp_query, nfexp_callback_register,\
    nfexp_callback_unregister, nfexp_catch,\
    NFCT_Q_DUMP, NFCT_Q_FLUSH, NFCT_T_ALL, EXPECT,\
    NFCT_CB_STOLEN
from pynetfilter_conntrack.ctypes_stdint import uint8_t
from ctypes import byref
from socket import AF_INET
from pynetfilter_conntrack.ctypes_errno import get_errno
from os import strerror
from pynetfilter_conntrack.conntrack_base import ConntrackBase

class Expect(ConntrackBase):
    def __init__(self, subscriptions=0, subsys=EXPECT):
        ConntrackBase.__init__(self, subsys, subscriptions)

    def register_callback(self, callback, event_type=NFCT_T_ALL, data=None):
        """
        Register a callback, needed by some query.
        Callback Python prototype is: func(msgtype, exp, data), C prototype:
           int cb(enum nf_conntrack_msg_type type,
                  struct nf_expect *exp,
                  void *data)

        The callback have to return: NFCT_CB_CONTINUE, NFCT_CB_FAILURE,
        NFCT_CB_STOP, or NFCT_CB_STOLEN (like continue, but ct is not freed).
        """
        self.callback = nfexp_callback_t(callback)
        self.callback_arg = data
        nfexp_callback_register(self.handle, event_type, self.callback, self.callback_arg)

    def unregister_callback(self):
        """Unregister callback"""
        nfexp_callback_unregister(self.handle)
        self.callback = None
        self.callback_arg = None

    def query(self, command, argument):
        """
        Do query libnetfilter_conntrack:
         - command: NFCT_Q_CREATE, NFCT_Q_UPDATE, ...
         - argument (optional): value depends on command

        May raise a RuntimeError.
        """
        ret = nfexp_query(self.handle, command, argument)
        if ret != 0:
            raise RuntimeError("nfct_query() failure: %s" % strerror(get_errno()))

    def dump_table(self, family=AF_INET, event_type=NFCT_T_ALL):
        # Create a pointer to a 'uint8_t' of the address family
        family = byref(uint8_t(family))

        def copyEntry(msgtype, ct, data):
            copyEntry.ctlist.append(ExpectEntry(self, ct, msgtype))
            return NFCT_CB_STOLEN
        copyEntry.ctlist = []

        # Install callback, do the query, remove callback
        self.register_callback(copyEntry, event_type)
        self.query(NFCT_Q_DUMP, family)
        self.unregister_callback()
        return copyEntry.ctlist

    def flush(self, family=AF_INET):
        family = byref(uint8_t(family))
        self.query(NFCT_Q_FLUSH, family)

    def catch(self, callback):
        """
        Catch all Netfilter expect events: call specified callback for
        each event. See register_callback() method for callback details.
        """
        self.register_callback(callback)
        ret = nfexp_catch(self.handle)
        if ret != 0:
            raise RuntimeError("nfexp_catch() failure: %s" % strerror(get_errno()))

__all__ = ('Expect', )




# conntrack attributes
ATTRIBUTES = {
    "orig_ipv4_src": (0, 32),
    "orig_ipv4_dst": (1, 32),
    "repl_ipv4_src": (2, 32),
    "repl_ipv4_dst": (3, 32),
    "orig_ipv6_src": (4, 128),
    "orig_ipv6_dst": (5, 128),
    "repl_ipv6_src": (6, 128),
    "repl_ipv6_dst": (7, 128),
    "orig_port_src": (8, 16),
    "orig_port_dst": (9, 16),
    "repl_port_src": (10, 16),
    "repl_port_dst": (11, 16),
    "icmp_type": (12, 8),
    "icmp_code": (13, 8),
    "icmp_id": (14, 8),
    "orig_l3proto": (15, 8),
    "repl_l3proto": (16, 8),
    "orig_l4proto": (17, 8),
    "repl_l4proto": (18, 8),
    "tcp_state": (19, 8),
    "snat_ipv4": (20, 32),
    "dnat_ipv4": (21, 32),
    "snat_port": (22, 16),
    "dnat_port": (23, 16),
    "timeout": (24, 32),
    "mark": (25, 32),
    "orig_counter_packets": (26, 64),
    "repl_counter_packets": (27, 64),
    "orig_counter_bytes": (28, 64),
    "repl_counter_bytes": (29, 64),
    "use": (30, 32),
    "id": (31, 32),
    "status": (32, 32),
}

# message type
NFCT_T_UNKNOWN = 0
NFCT_T_NEW = (1 << 0)
NFCT_T_UPDATE = (1 << 1)
NFCT_T_DESTROY = (1 << 2)
NFCT_T_ALL = NFCT_T_NEW | NFCT_T_UPDATE | NFCT_T_DESTROY
NFCT_T_ERROR = (1 << 31)

# set option
NFCT_SOPT_UNDO_SNAT = 0
NFCT_SOPT_UNDO_DNAT = 1
NFCT_SOPT_UNDO_SPAT = 2
NFCT_SOPT_UNDO_DPAT = 3
NFCT_SOPT_MAX = NFCT_SOPT_UNDO_DPAT

# get option
NFCT_GOPT_IS_SNAT = 0
NFCT_GOPT_IS_DNAT = 1
NFCT_GOPT_IS_SPAT = 2
NFCT_GOPT_IS_DPAT = 3
NFCT_GOPT_MAX = NFCT_GOPT_IS_DPAT

# callback verdict
NFCT_CB_FAILURE = -1    # failure
NFCT_CB_STOP = 0        # stop the query
NFCT_CB_CONTINUE = 1    # keep iterating through data
NFCT_CB_STOLEN = 2      # like continue, but ct is not freed

# output type
NFCT_O_DEFAULT = 0
NFCT_O_XML = 1

# output flags
NFCT_OF_SHOW_LAYER3 = (1 << 0)

# compare flags
NFCT_MARK = (1 << 3)

# query
NFCT_Q_CREATE = 0
NFCT_Q_UPDATE = 1
NFCT_Q_DESTROY = 2
NFCT_Q_GET = 3
NFCT_Q_FLUSH = 4
NFCT_Q_DUMP = 5
NFCT_Q_DUMP_RESET = 6

# layer 3 protocol families
PF_UNSPEC = 0     # Unspecified
PF_LOCAL = 1      # Local to host (pipes and file-domain)
PF_UNIX = 1       # Old BSD name for PF_LOCAL
PF_FILE = 1       # Another non-standard name for PF_LOCAL
PF_INET = 2       # IP protocol family
PF_AX25 = 3       # Amateur Radio AX.25
PF_IPX = 4        # Novell Internet Protocol
PF_APPLETALK = 5  # Appletalk DDP
PF_NETROM = 6     # Amateur radio NetROM
PF_BRIDGE = 7     # Multiprotocol bridge
PF_ATMPVC = 8     # ATM PVCs
PF_X25 = 9        # Reserved for X.25 project
PF_INET6 = 10     # IP version 6
PF_ROSE = 11      # Amateur Radio X.25 PLP
PF_DECnet = 12    # Reserved for DECnet project
PF_NETBEUI = 13   # Reserved for 802.2LLC project
PF_SECURITY = 14  # Security callback pseudo AF
PF_KEY = 15       # PF_KEY key management API
PF_NETLINK = 16
PF_ROUTE = 16     # Alias to emulate 4.4BSD
PF_PACKET = 17    # Packet family
PF_ASH = 18       # Ash
PF_ECONET = 19    # Acorn Econet
PF_ATMSVC = 20    # ATM SVCs
PF_SNA = 22       # Linux SNA Project
PF_IRDA = 23      # IRDA sockets
PF_PPPOX = 24     # PPPoX sockets
PF_WANPIPE = 25   # Wanpipe API sockets
PF_BLUETOOTH = 31 # Bluetooth sockets
PF_MAX = 32       # For now...

# layer 4 protocols
IPPROTO_IP = 0          # Dummy protocol for TCP
IPPROTO_HOPOPTS = 0     # IPv6 Hop-by-Hop options
IPPROTO_ICMP = 1        # Internet Control Message Protocol
IPPROTO_IGMP = 2        # Internet Group Management Protocol
IPPROTO_IPIP = 4        # IPIP tunnels (older KA9Q tunnels use 94)
IPPROTO_TCP = 6         # Transmission Control Protocol
IPPROTO_EGP = 8         # Exterior Gateway Protocol
IPPROTO_PUP = 12        # PUP protocol
IPPROTO_UDP = 17        # User Datagram Protocol
IPPROTO_IDP = 22        # XNS IDP protocol
IPPROTO_TP = 29         # SO Transport Protocol Class 4
IPPROTO_IPV6 = 41       # IPv6-in-IPv4 tunnelling
IPPROTO_ROUTING = 43    # IPv6 routing header
IPPROTO_FRAGMENT = 44,  # IPv6 fragmentation header
IPPROTO_RSVP = 46       # RSVP protocol
IPPROTO_GRE = 47        # Cisco GRE tunnels (rfc 1701,1702)
IPPROTO_ESP = 50        # Encapsulation Security Payload protocol
IPPROTO_AH = 51         # Authentication Header protocol
IPPROTO_ICMPV6 = 58     # ICMPv6
IPPROTO_NONE = 59       # IPv6 no next header
IPPROTO_DSTOPTS = 60    # IPv6 destination options
IPPROTO_MTP = 92        # Multicast Transport Protocol
IPPROTO_ENCAP = 98      # Encapsulation Header
IPPROTO_PIM = 103       # Protocol Independent Multicast
IPPROTO_COMP = 108      # Compression Header protocol
IPPROTO_SCTP = 132      # Stream Control Transport Protocol
IPPROTO_RAW = 255

# tcp state
NFCT_TCP_ST_NONE = 0
NFCT_TCP_ST_SYN_SENT = 1
NFCT_TCP_ST_SYN_RECV = 2
NFCT_TCP_ST_ESTABLISHED = 3
NFCT_TCP_ST_FIN_WAIT = 4
NFCT_TCP_ST_CLOSE_WAIT = 5
NFCT_TCP_ST_LAST_ACK = 6
NFCT_TCP_ST_TIME_WAIT = 7
NFCT_TCP_ST_CLOSE = 8
NFCT_TCP_ST_LISTEN = 9

# Netfilter subsystem identifier (libnfnetlink/linux_nfnetlink.h)
CONNTRACK = NFNL_SUBSYS_CTNETLINK = 1
EXPECT = NFNL_SUBSYS_CTNETLINK_EXP = 2


### Bitset representing status of connection ################################

IPS_EXPECTED = (1 << 0)

# We've seen packets both ways: bit 1 set.  Can be set, not unset.
IPS_SEEN_REPLY = (1 << 1)

# Conntrack should never be early-expired.
IPS_ASSURED = (1 << 2)

# Connection is confirmed: originating packet has left box
IPS_CONFIRMED = (1 << 3)

# Connection needs src nat in orig dir.  This bit never changed.
IPS_SRC_NAT = (1 << 4)

# Connection needs dst nat in orig dir.  This bit never changed.
IPS_DST_NAT = (1 << 5)

# Both together.
IPS_NAT_MASK = (IPS_DST_NAT | IPS_SRC_NAT)

# Connection needs TCP sequence adjusted.
IPS_SEQ_ADJUST = (1 << 6)

# NAT initialization bits.
IPS_SRC_NAT_DONE = (1 << 7)
IPS_DST_NAT_DONE = (1 << 8)

# Both together
IPS_NAT_DONE_MASK = (IPS_DST_NAT_DONE | IPS_SRC_NAT_DONE)

# Connection is dying (removed from lists), can not be unset.
IPS_DYING = (1 << 9)

# Connection has fixed timeout.
IPS_FIXED_TIMEOUT = (1 << 10)

# nfnetlink groups: Up to 32 maximum - backwards compatibility for userspace
NF_NETLINK_CONNTRACK_NEW         = 0x00000001
NF_NETLINK_CONNTRACK_UPDATE      = 0x00000002
NF_NETLINK_CONNTRACK_DESTROY     = 0x00000004
NF_NETLINK_CONNTRACK_EXP_NEW     = 0x00000008
NF_NETLINK_CONNTRACK_EXP_UPDATE  = 0x00000010
NF_NETLINK_CONNTRACK_EXP_DESTROY = 0x00000020
NFCT_ALL_CT_GROUPS = (NF_NETLINK_CONNTRACK_NEW \
    | NF_NETLINK_CONNTRACK_UPDATE | NF_NETLINK_CONNTRACK_DESTROY)

##__all__ = (
##    "ATTRIBUTES",
##    "NFCT_Q_CREATE", "NFCT_Q_UPDATE", "NFCT_Q_DESTROY", "NFCT_Q_GET",
##    "NFCT_Q_FLUSH", "NFCT_Q_DUMP", "NFCT_Q_DUMP_RESET",
##    "NFCT_T_UNKNOWN", "NFCT_T_NEW", "NFCT_T_UPDATE", "NFCT_T_DESTROY",
##    "NFCT_T_ALL", "NFCT_T_ERROR",
##    "CONNTRACK", "EXPECT",
##    "NFCT_OF_SHOW_LAYER3", "NFCT_O_DEFAULT", "NFCT_O_XML",
##    "NFCT_CB_FAILURE", "NFCT_CB_STOP", "NFCT_CB_CONTINUE", "NFCT_CB_STOLEN",
##    "IPS_EXPECTED", "IPS_SEEN_REPLY", "IPS_ASSURED", "IPS_CONFIRMED",
##    "IPS_SRC_NAT", "IPS_DST_NAT", "IPS_NAT_MASK", "IPS_SEQ_ADJUST",
##    "IPS_SRC_NAT_DONE", "IPS_DST_NAT_DONE", "IPS_NAT_DONE_MASK",
##    "IPS_DYING", "IPS_FIXED_TIMEOUT",
##    "PF_UNSPEC", "PF_LOCAL", "PF_UNIX", "PF_FILE", "PF_INET", "PF_AX25",
##    "PF_IPX", "PF_APPLETALK", "PF_NETROM", "PF_BRIDGE", "PF_ATMPVC", "PF_X25",
##    "PF_INET6", "PF_ROSE", "PF_DECnet", "PF_NETBEUI", "PF_SECURITY", "PF_KEY",
##    "PF_NETLINK", "PF_ROUTE", "PF_PACKET", "PF_ASH", "PF_ECONET", "PF_ATMSVC",
##    "PF_SNA", "PF_IRDA", "PF_PPPOX", "PF_WANPIPE", "PF_BLUETOOTH", "PF_MAX",
##    "IPPROTO_IP", "IPPROTO_HOPOPTS", "IPPROTO_ICMP", "IPPROTO_IGMP",
##    "IPPROTO_IPIP", "IPPROTO_TCP", "IPPROTO_EGP", "IPPROTO_PUP", "IPPROTO_UDP",
##    "IPPROTO_IDP", "IPPROTO_TP", "IPPROTO_IPV6", "IPPROTO_ROUTING",
##    "IPPROTO_FRAGMENT", "IPPROTO_RSVP", "IPPROTO_GRE", "IPPROTO_ESP",
##    "IPPROTO_AH", "IPPROTO_ICMPV6", "IPPROTO_NONE", "IPPROTO_DSTOPTS",
##    "IPPROTO_MTP", "IPPROTO_ENCAP", "IPPROTO_PIM", "IPPROTO_COMP",
##    "IPPROTO_SCTP", "IPPROTO_RAW",
##)

# tcp_state
TCP_CONNTRACK_NONE = 0
TCP_CONNTRACK_SYN_SENT = 1
TCP_CONNTRACK_SYN_RECV = 2
TCP_CONNTRACK_ESTABLISHED = 3
TCP_CONNTRACK_FIN_WAIT = 4
TCP_CONNTRACK_CLOSE_WAIT = 5
TCP_CONNTRACK_LAST_ACK = 6
TCP_CONNTRACK_TIME_WAIT = 7
TCP_CONNTRACK_CLOSE = 8
TCP_CONNTRACK_LISTEN = 9
TCP_CONNTRACK_MAX = 10
TCP_CONNTRACK_IGNORE = 11

# set option
NFCT_SOPT_UNDO_SNAT = 0
NFCT_SOPT_UNDO_DNAT = 1
NFCT_SOPT_UNDO_SPAT = 2
NFCT_SOPT_UNDO_DPAT = 3
NFCT_SOPT_SETUP_ORIGINAL = 4
NFCT_SOPT_SETUP_REPLY = 5




from ctypes import sizeof, \
    c_byte, c_ubyte, \
    c_short, c_ushort, \
    c_int, c_uint, \
    c_long, c_ulong, \
    c_longlong, c_ulonglong

#------------- ctypes types: uint8_t, int16_t, size_t, ... ---------------

# 8-bit integers
uint8_t = c_ubyte
int8_t = c_byte

# 16-bit integers
assert sizeof(c_short) == 2
assert sizeof(c_ushort) == 2
int16_t = c_short
uint16_t = c_ushort

# 32-bit integers
assert sizeof(c_int) == 4
assert sizeof(c_uint) == 4
int32_t = c_int
uint32_t = c_uint

# 64-bit integers
if sizeof(c_long) == 8:
    int64_t = c_long
else:
    assert sizeof(c_longlong) == 8
    int64_t = c_longlong
if sizeof(c_ulong) == 8:
    uint64_t = c_ulong
else:
    assert sizeof(c_ulonglong) == 8
    uint64_t = c_ulonglong

# size_t
size_t = c_long

__all__ = (
    "uint8_t", "int8_t", "int16_t", "uint16_t",
    "int32_t", "uint32_t", "int64_t", "uint64_t", "size_t")




from ctypes import string_at
from struct import pack, unpack
from sys import hexversion

def isBigEndian():
    """
    Get machine endian: True for big endian, False for little endian
    """
    native_value = 0x1234   # 16-bit value
    native_str = pack("@H", native_value)
    bigendian_value = unpack(">H", native_str)[0]
    return (bigendian_value == native_value)

def raw2long(data, big_endian):
    r"""
    Convert a raw data (type 'str') into a long integer.

    >>> chr(raw2long('*', True))
    '*'
    >>> raw2long("\x00\x01\x02\x03", True) == 0x10203
    True
    >>> raw2long("\x2a\x10", False) == 0x102a
    True
    >>> raw2long("\xff\x14\x2a\x10", True) == 0xff142a10L
    True
    >>> raw2long("\x00\x01\x02\x03", False) == 0x3020100
    True
    >>> raw2long("\xff\x14\x2a\x10\xab\x00\xd9\x0e", True) == 0xff142a10ab00d90eL
    True
    >>> raw2long("\xff\xff\xff\xff\xff\xff\xff\xfe", True) == (2**64-2)
    True
    """
    assert 1 <= len(data) <= 16   # arbitrary limit: 8..128 bits
    if big_endian:
        INDEXES = xrange(len(data)-1,-1,-1)
    else:
        INDEXES = xrange(len(data))
    shift = 0L
    value = 0
    for index in INDEXES:
        byte = ord(data[index])
        value += (byte << shift)
        shift += 8
    return value

def ctypes_ptr2uint(ptr, size):
    """
    Read unsigned integer of 'size' bytes at 'ptr' address where 'ptr' is
    a ctypes 'c_void_p' pointer. Use machine endian.
    """
    raw = string_at(ptr, size)
    return raw2long(raw, ctypes_ptr2uint.big_endian)
ctypes_ptr2uint.big_endian = isBigEndian()

def __int16_to_uint16_old(n):
    if n < 0:
        return 0x10000 + n
    else:
        return n

def __int32_to_uint32_old(n):
    if n < 0:
        return 0x100000000L + n
    else:
        return long(n)

def __int16_to_uint16_new(n):
    return n & 0xFFFF

def __int32_to_uint32_new(n):
    return n & 0xFFFFFFFFL

if hexversion < 0x2040000:
    int16_to_uint16 = __int16_to_uint16_old
    int32_to_uint32 = __int32_to_uint32_old
else:
    int16_to_uint16 = __int16_to_uint16_new
    int32_to_uint32 = __int32_to_uint32_new

int16_to_uint16.__doc__ = """Convert a 16 bits signed integer to unsigned integer.
Examples:

>>> int16_to_uint16(10627)
10627
>>> int16_to_uint16(-10627)
54909
"""

int32_to_uint32.__doc__ = """Convert a 32 bits signed integer to unsigned integer.
Examples:

>>> int32_to_uint32(1062723156)
1062723156L
>>> int32_to_uint32(-1062723156)
3232244140L
"""

__all__ = ("raw2long", "ctypes_ptr2uint", "int16_to_uint16", "int32_to_uint32")




#!/usr/bin/env python
# Installer: use --setuptools to use setuptools

from os import path
import sys
from imp import load_source
if "--setuptools" in sys.argv:
    sys.argv.remove("--setuptools")
    from setuptools import setup
    use_setuptools = True
else:
    from distutils.core import setup
    use_setuptools = False

# Retrieve revision
pynetfilter_conntrack = load_source("version", path.join("pynetfilter_conntrack", "version.py"))

DESCRIPTION = "pynetfilter_conntrack is a Python binding of libnetfilter_conntrack"
LONG_DESCRIPTION = open("README.rst").read() + open("INSTALL").read() + open("ChangeLog").read()
KEYWORDS = "netfilter conntrack ctypes firewall"
REQUIRES = ("ctypes>=0.9.6", "IPy>=0.50")

CLASSIFIERS = filter(None, map(str.strip,
"""
Development Status :: 4 - Beta
Natural Language :: English
Environment :: Console
Intended Audience :: Developers
Intended Audience :: System Administrators
License :: OSI Approved :: GNU General Public License (GPL)
Operating System :: POSIX :: Linux
Programming Language :: Python
Topic :: Software Development :: Libraries :: Python Modules
Topic :: System :: Networking :: Firewalls
Topic :: System :: Networking :: Monitoring
""".splitlines()))

option = {}
if use_setuptools:
    option["zip_safe"] = True
    option["install_requires"] = REQUIRES

setup(
    name=pynetfilter_conntrack.PACKAGE,
    version=pynetfilter_conntrack.VERSION,
    url=pynetfilter_conntrack.WEBSITE,
    download_url=pynetfilter_conntrack.WEBSITE,
    license=pynetfilter_conntrack.LICENSE,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    classifiers=CLASSIFIERS,
    author="Victor Stinner",
    author_email="victor.stinner AT inl.fr",
    keywords=KEYWORDS,
    packages=["pynetfilter_conntrack"],
    platforms=['Linux'],
    package_dir={'pynetfilter_conntrack': 'pynetfilter_conntrack'},
    scripts=["conntrack.py"],
    **option
)




from pynetfilter_conntrack.version import VERSION as __revision__
from pynetfilter_conntrack.tools import *
from pynetfilter_conntrack.constant import *
from pynetfilter_conntrack.func import *
from pynetfilter_conntrack.func_expect import *
from pynetfilter_conntrack.conntrack_entry import *
from pynetfilter_conntrack.filter import Filter
from pynetfilter_conntrack.conntrack import *
from pynetfilter_conntrack.expect_entry import *
from pynetfilter_conntrack.expect import *

